
/*
  GoodFields 10_clavier_analogique
  Clavier analogique à diodes et debug
  LCD
  Gestion du temps
  Compactage des sorties série (intégration d'une bibliothèque : Streaming)
  GRAFCET
  TEMPORISATION basique
  FRONT MONTANT/DESCENDANT
*/
//**********************************************************
// BIBLIOTHEQUES
// incluses dans le package Arduino de base
#include <LiquidCrystal.h>  // LCD

// ajoutées
#include <Streaming.h>
#include <Time.h>

// DEBUG : 0=pas de debug, valeur selon besoins
#define debug 1

//**********************************************************
// DECLARATIONS
// LCD initialize the library with the numbers of the interface pins
/* Câblage du LCD
 * LCD pin 1 to GND
 * LCD pin 2 to VCC 5V
 * LCD pin 3 : contraste sur le point milieu d'un potentiomètre de 10K entre le GND et le VCC (réglé très près de la masse)
 * LCD pin 4 RS to digital pin 4
 * LCD pin 5 RW to GND
 * LCD pin 6 Enable pin to digital pin 2
 * LCD pin 11 D4 pin to digital pin 6
 * LCD pin 12 D5 pin to digital pin 7
 * LCD pin 13 D6 pin to digital pin 8
 * LCD pin 14 D7 pin to digital pin 9
 
Les POWERTIP (grands non retro éclairés) démarrent très lentement.
J'ai tenté des modifs dans la bibliothèque sans succès.
Il suffit de faire un reset hard de l'arduino après mise sous tension du bazar
// F1FWG : http://www.powertipusa.com/ps.htm
*/

// entrées-sorties
int led_a = 13;

//  LiquidCrystal lcd(RS, Enable,  D4, D5, D6, D7)
  LiquidCrystal lcd(4, 2, 6, 7, 8, 9);

// temporisations
int tempo_a,tempo_b;
// vitesse de clignotement : maintenant en millisecondes
int tempo_a_reset=1000;
int tempo_b_reset=700;
// gestion du temps
int previousSecond;      // secondes de l'horloge
int top_dixieme=false;   // top pour traitements au 1/10 seconde dans le P/P
int top_seconde=false;   // top pour traitements à la seconde dans le P/P
int top_minute=false;   // top pour traitements à la minute dans le P/P
unsigned long previousMillis, previousDecis;
// clavier à 16 touches et diodes sur entrée analogique
int clavier=A0;
int itouche,ntouche,ntouche1=-1,ntouche2=-1,vtouche;
// touche renvoyée
char touche=' ',touche_repeat=' ',tempo_repeat;

// CLAVIER 10 TOUCHES A DIODES 0,2V
int nbtouche = 10;
// caractère touche renvoyée
//                 <----------------clavier-10 touches------------>                16 touches--->                          24 touches--->
char ltouche[] =  {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ' '};        // nbtouche
// valeurs analogiques. A ajuster selon valeurs relevées après montage # 41 pour 0,2 Volts/diode
int xtouche[] =   {  0,  45,  90, 136, 181, 226, 270, 313, 354, 396, 437, 1023};  // nbtouche+1

/*
// CLAVIER 24 TOUCHES A DIODES 0,2V
int nbtouche = 24;
// caractère touche renvoyée
//                 <----------------clavier-10 touches------------>                16 touches--->                          24 touches--->
char ltouche[] =  {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', ' '};        // nbtouche
// valeurs analogiques. A ajuster selon valeurs relevées après montage # 41 pour 0,2 Volts/diode
int xtouche[] =   {  0,  45,  91, 136, 181, 226, 270, 313, 354, 396, 437, 478, 519, 560, 601, 642, 683, 724, 765, 806, 847, 888, 929, 970, 1010, 1023};  // nbtouche+1
*/

// speedomètre = calcul de vitesse ...
int tours_ps,tours;

// GRAFCETS
int grafcet_a,grafcet_b;

//**********************************************************
// INITIALISATIONS
void setup() {
// définition taille du LCD :
  lcd.begin(16, 2);

// mise à l'heure
  setTime(12,15,00,25,12,13); // 12h15 le 25 déc 2013 setupArduinoTime(2013,25,21,12,15,00));
// initialize the pins
  pinMode(led_a, OUTPUT);
  pinMode(clavier, INPUT);
  digitalWrite(clavier, HIGH);       // turn on pullup resistors

// ligne série (pour traces)
  Serial.begin(9600);  // ou 9600 selon votre port COM
#if debug > 0  
  Serial << "Serial ok" << endl;
#endif

// AFFICHAGE sur LCD
  lcd.setCursor(0, 0);  lcd << "Serial OK";     // 1ere ligne
  lcd.setCursor(0, 1);  lcd << "GoodFields";    // 2eme ligne lcd.print ("toto="); lcd.pprint (toto,2);
  delay(1000);        // pour profiter du LCD au démarrage
  lcd.clear();    // Effacement général LCD
  
// on arme les grafcets
  grafcet_a = 1;
  grafcet_b = 1;
}
//**********************************************************
// PROGRAMME PRINCIPAL
void loop() {

// tours par seconde
  tours++;
  
// gestion du temps
  ger_top_centieme ();
  
// opérations à chaque dixième de seconde
  if (top_dixieme) {
    top_dixieme = false;
/*
// premier test
    itouche = analogRead(clavier);
    itouche1 = (itouche+20)/44;
    lcd.setCursor(0, 1);  lcd << "t=" << itouche1 << "=" << itouche << "  ";    // 2eme ligne lcd
//    lcd.setCursor(0, 1);  lcd << "b=" << grafcet_b << " a=" << grafcet_a ;    // 2eme ligne lcd
*/
  }
 
// opérations à chaque seconde
  if (top_seconde) {
    top_seconde = false;
    tours_ps=tours; tours=0;
#if debug > 0  
    lcd.setCursor(12, 1);  lcd << tours_ps ; //<< "t/s  ";    // 2eme ligne lcd
#endif

// horloge
#if debug > 0  
    horodatage_serial(); Serial << " top_seconde " << tours_ps << "t/s  " << endl;
#endif
    lcd.setCursor(0, 0);
    if (second()%10 > 7) {
      lcd << "XXX " << ((day()<10)?"0":"") << day() << "/" << ((month()<10)?"0":"") << month() << "/" << year();
    } else {
      lcd << "    " << ((hour()<10)?"0":"") << hour() << ":" << ((minute()<10)?"0":"") << minute() << ":" << ((second()<10)?"0":"") << second() << "    ";
    }
// idéalement, XXX devrait être remplacé par {"Lun","Mar","Mer","Jeu","Ven","Sam","Dim"} (au boulot !)
    
// heart-beat chaque seconde   
//    if (lamp_tempo == 0 ) lamp_tempo = 20;  // ms
    }

//--------------------------------
// GRAFCET A
switch (grafcet_a) {

  case 1:
  if ( tempo_a == 0 ) {        // si la tempo est terminée,
    digitalWrite(led_a, HIGH);   // on allume la LED
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a++;               // et on passe à l'étape suivante
  }
  break;      // cette instruction vous fait sauter à la fin du "switch"
  // sinon, on continue, et cela fait perdre du temps, et il peut arriver que d'autre "case" fonctionnent ! 

  case 2:
  if ( tempo_a == 0 ) {
    digitalWrite(led_a, LOW);    // turn the LED off by making the voltage LOW
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a=1;               // on reboucle
  }    
  break;

// si grafcet_a n'a pas une valeur correcte, on passe ici
  default: 
      grafcet_a = 0; // on arrête
  break;
}      // fin du "switch"
// FIN GRAFCET A
//--------------------------------
// GRAFCET B
switch (grafcet_b) {

  case 1:
  if ( tempo_b == 0 ) {        // si la tempo est terminée,
    horodatage_serial(); Serial << " grafcet_b=" << grafcet_b <<  " grafcet_a=" << grafcet_a <<endl;
    tempo_b = tempo_b_reset;             // on arme la tempo
    grafcet_b++;               // et on passe à l'étape suivante
  }
  break;      // cette instruction vous fait sauter à la fin du "switch"

  case 2:
  if ( tempo_b == 0 ) {
    horodatage_serial(); Serial << " grafcet_b=" << grafcet_b <<  " grafcet_a=" << grafcet_a << endl;
    tempo_b = tempo_b_reset;             // on arme la tempo
    grafcet_b=1;               // on reboucle
  }    
  break;

// si grafcet_a n'a pas une valeur correcte, on passe ici
  default: 
      grafcet_b = 0; // on arrête
  break;
}      // fin du "switch"
// FIN GRAFCET B
//--------------------------------
}
// ************************************************************************************************
// ************************************************************************************************
// SOUS-PROGRAMMES
// ************************************************************************************************
// ************************************************************************************************
/* scrutation et filtrage de clavier analogique

  utilisation "monocoup" :
  if (touche == '1') { ...; touche=' ';}

  utilisation avec "répétition" :
  if (touche_repeat == '1') { ...; touche_repeat=' ';}
*/

void ger_scrute_clavier(){
// clavier - filtrage sur NO de touche (c'est plus stable que sur valeur)
  ntouche2 = ntouche1;
  ntouche1 = ntouche;

  itouche = analogRead(clavier);

    ntouche=-1;
    do {
      ntouche++;
    } while (itouche > ((xtouche[ntouche]+xtouche[ntouche+1])/2) && ntouche < nbtouche);
// OU en appliquant la formule n = ( valeur_ana + 20 ) / 41 on devrait récupérer le No de la touche de 0 à 9 ...

// TRACE pour calibrage
//  if (itouche < 1000 ) Serial << itouche << "=" << ntouche << endl;

// si la touche est stable (X tours)
  if (ntouche==ntouche1 && ntouche==ntouche2) {

// si la touche a varié
    if (ltouche[ntouche] != vtouche ) {
//    if (itouche < 1000 ) Serial << itouche << "=" << ntouche << endl;
      vtouche = ltouche[ntouche];
// touche valide
      if (ntouche < nbtouche) { 
        touche = vtouche;
        touche_repeat = vtouche;
        tempo_repeat=800;  // duree avant repetition
// arret répétition
      } else {
        touche_repeat = ' ';
      }
// si la touche est la même : repeat ?
    } else {
      if (ntouche < nbtouche && tempo_repeat == 0 ) {
        touche_repeat = vtouche;
        tempo_repeat=500;  // vitesse repetition
      }
    }
// debug
//  } else {
//  Serial << itouche6 << " " << itouche5 << " " << itouche4 << " " << itouche3 << " " << itouche2 << " " << itouche1 << " " << itouche << endl;
  }
  return;
}

// ************************************************************************************************
// Programme au -ieme de seconde
// gestion des temporisations
// NB : le débordement du compteur en (unsigned long) a lieu tous les 50 jours environ ...
void ger_top_centieme (){
  unsigned long delta;
  unsigned long currentMillis = millis();

// Centième de secondes
   delta = currentMillis - previousMillis;
   if (delta >= 10) {
// save the last time you come here
    previousMillis = currentMillis;   

// scrutation du clavier toutes les 10ms
    ger_scrute_clavier();
// Donc en appliquant la formule n = ( valeur_ana + 20 ) / 41 on devrait récupérer le No de la touche de 0 à 9 ...

// allumage temporisé led clavier
//    if (lamp_tempo >= delta) { lamp_tempo -= delta; digitalWrite(lamp, HIGH); } else { lamp_tempo = 0; digitalWrite(lamp, LOW); }

// temporisations diverses
//    if (fonction_tempo >= delta) { fonction_tempo -= delta; } else { fonction_tempo = 0; }
// gestion des TEMPORISATIONS
    if (tempo_a >= delta ) { tempo_a -= delta; } else { tempo_a = 0; }
    if (tempo_b >= delta ) { tempo_b -= delta; } else { tempo_b = 0; }
    if (tempo_repeat >= delta ) { tempo_repeat -= delta; } else { tempo_repeat = 0; }
   }

// Dixième de seconde
   delta = currentMillis - previousDecis;
   if (delta >= 100) {
    previousDecis = currentMillis;   
    top_dixieme = true;
    
// tops seconde et minute pour traitements dans le main
    if (second() != previousSecond) {
      previousSecond = second();
      top_seconde = true;
      if (previousSecond == 0) top_minute=true;
    }
  }
  return;
}
// ************************************************************************************************
// affichage horloge complete sur la sortie serie, sans fin de ligne
void horodatage_serial() {
// digital clock display of the time
//  Serial << day() << "/" << month() << "/" << year() << " " << hour() << printDigits(minute()) << printDigits(second()) << endl; 
  Serial << ((day()<10)?"0":"") << day() << "/" << ((month()<10)?"0":"") << month() << "/" << year() << " " << ((hour()<10)?"0":"") << hour() << ":" << ((minute()<10)?"0":"") << minute() << ":" << ((second()<10)?"0":"") << second();
}
// ************************************************************************************************
/*
// Transforme la date et l'heure "lisible" en format UNIX pour mise à l'heure et date
time_t setupArduinoTime(int y,int m, int j, int h, int mi, int se) {
  tmElements_t tm;
  tm.Year = y-1970;
  tm.Month = m;
  tm.Day = j;
  tm.Hour = h;
  tm.Minute = mi;
  tm.Second = se;
  time_t time = makeTime(tm);
  return time;
}
//+ voici le s/p original qui provient d'une bibliothèque pour GPS :
time_t gpsTimeToArduinoTime(){
// returns time_t from gps date and time with the given offset hours
  tmElements_t tm;
  int year;
  gps.crack_datetime(&year, &tm.Month, &tm.Day, &tm.Hour, &tm.Minute, &tm.Second, NULL, NULL);
  time_t time = makeTime(tm);
  return time + (offset * SECS_PER_HOUR);
}
*/
// ************************************************************************************************

